#include "TypeCheck.h"
#include <stdexcept>

void TypeCheck::visitProgram(Program *t) {} //abstract class
void TypeCheck::visitTopDef(TopDef *t) {} //abstract class
void TypeCheck::visitFunDef(FunDef *t) {} //abstract class
void TypeCheck::visitArg(Arg *t) {} //abstract class
void TypeCheck::visitClassDef(ClassDef *t) {
  t->getName()->accept(this);
  if (t->getParent())
  	t->getParent()->accept(this);
  t->getBlock()->accept(this);
}

void TypeCheck::visitClassBlock(ClassBlock *t) {} //abstract class
void TypeCheck::visitClassBlockDef(ClassBlockDef *t) {} //abstract class
void TypeCheck::visitBlock(Block *t) {} //abstract class
void TypeCheck::visitStmt(Stmt *t) {} //abstract class
void TypeCheck::visitItem(Item *t) {} //abstract class
void TypeCheck::visitType(Type *t) {} //abstract class
void TypeCheck::visitExpr(Expr *t) {} //abstract class
void TypeCheck::visitAddOp(AddOp *t) {} //abstract class
void TypeCheck::visitMulOp(MulOp *t) {} //abstract class
void TypeCheck::visitRelOp(RelOp *t) {} //abstract class

void TypeCheck::visitPIdent(PIdent *p_ident)
{
  /* Code For PIdent Goes Here */

  visitString(p_ident->string_);
  visitInteger(p_ident->integer_);

}

void TypeCheck::visitProg(Prog *prog)
{
  /* Code For Prog Goes Here */

  prog->listtopdef_->accept(this);

}

void TypeCheck::visitFnDef(FnDef *fn_def)
{
  /* Code For FnDef Goes Here */

  fn_def->fundef_->accept(this);

}

void TypeCheck::visitClDef(ClDef *cl_def)
{
  /* Code For ClDef Goes Here */

  cl_def->classdef_->accept(this);

}

void TypeCheck::visitFuncDef(FuncDef *func_def)
{
  /* Code For FuncDef Goes Here */

  func_def->type_->accept(this);
  func_def->pident_->accept(this);
  func_def->listarg_->accept(this);
  func_def->block_->accept(this);

}

void TypeCheck::visitAr(Ar *ar)
{
  /* Code For Ar Goes Here */

  ar->type_->accept(this);
  ar->pident_->accept(this);

}

void TypeCheck::visitClassDefN(ClassDefN *class_def_n)
{
  visitClassDef(class_def_n);
}

void TypeCheck::visitClassDefE(ClassDefE *class_def_e)
{
  visitClassDef(class_def_e);
}

void TypeCheck::visitClassBl(ClassBl *class_bl)
{
  /* Code For ClassBl Goes Here */

  class_bl->listclassblockdef_->accept(this);

}

void TypeCheck::visitClassMthd(ClassMthd *class_mthd)
{
  /* Code For ClassMthd Goes Here */

  class_mthd->fundef_->accept(this);

}

void TypeCheck::visitClassFld(ClassFld *class_fld)
{
  /* Code For ClassFld Goes Here */

  class_fld->type_->accept(this);
  class_fld->listitem_->accept(this);

}

void TypeCheck::visitBlk(Blk *blk)
{
  /* Code For Blk Goes Here */

  blk->liststmt_->accept(this);

}

void TypeCheck::visitEmpty(Empty *empty)
{
  /* Code For Empty Goes Here */


}

void TypeCheck::visitBStmt(BStmt *b_stmt)
{
  /* Code For BStmt Goes Here */

  b_stmt->block_->accept(this);

}

void TypeCheck::visitDecl(Decl *decl)
{
  /* Code For Decl Goes Here */

  decl->type_->accept(this);
  decl->listitem_->accept(this);

}

void TypeCheck::visitAss(Ass *ass)
{
  /* Code For Ass Goes Here */

  ass->expr_1->accept(this);
  ass->expr_2->accept(this);

}

void TypeCheck::visitTableAss(TableAss *table_ass)
{
  /* Code For TableAss Goes Here */

  table_ass->pident_->accept(this);
  table_ass->expr_1->accept(this);
  table_ass->expr_2->accept(this);

}

void TypeCheck::visitTableIncr(TableIncr *table_incr)
{
  /* Code For TableIncr Goes Here */

  table_incr->pident_->accept(this);
  table_incr->expr_->accept(this);

}

void TypeCheck::visitTableDecr(TableDecr *table_decr)
{
  /* Code For TableDecr Goes Here */

  table_decr->pident_->accept(this);
  table_decr->expr_->accept(this);

}

void TypeCheck::visitIncr(Incr *incr)
{
  /* Code For Incr Goes Here */

  incr->pident_->accept(this);

}

void TypeCheck::visitDecr(Decr *decr)
{
  /* Code For Decr Goes Here */

  decr->pident_->accept(this);

}

void TypeCheck::visitRet(Ret *ret)
{
  /* Code For Ret Goes Here */

  ret->expr_->accept(this);

}

void TypeCheck::visitVRet(VRet *v_ret)
{
  /* Code For VRet Goes Here */


}

void TypeCheck::visitCond(Cond *cond)
{
  /* Code For Cond Goes Here */

  cond->expr_->accept(this);
  cond->stmt_->accept(this);

}

void TypeCheck::visitCondElse(CondElse *cond_else)
{
  /* Code For CondElse Goes Here */

  cond_else->expr_->accept(this);
  cond_else->stmt_1->accept(this);
  cond_else->stmt_2->accept(this);

}

void TypeCheck::visitWhile(While *while_)
{
  /* Code For While Goes Here */

  while_->expr_->accept(this);
  while_->stmt_->accept(this);

}

void TypeCheck::visitSExp(SExp *s_exp)
{
  /* Code For SExp Goes Here */

  s_exp->expr_->accept(this);

}

void TypeCheck::visitForEach(ForEach *for_each)
{
  /* Code For ForEach Goes Here */

  for_each->type_->accept(this);
  for_each->pident_->accept(this);
  for_each->expr_->accept(this);
  for_each->stmt_->accept(this);

}

void TypeCheck::visitNoInit(NoInit *no_init)
{
  /* Code For NoInit Goes Here */

  no_init->pident_->accept(this);

}

void TypeCheck::visitInit(Init *init)
{
  /* Code For Init Goes Here */

  init->pident_->accept(this);
  init->expr_->accept(this);

}

void TypeCheck::visitInt(Int *int_)
{
  /* Code For Int Goes Here */


}

void TypeCheck::visitStr(Str *str)
{
  /* Code For Str Goes Here */


}

void TypeCheck::visitBool(Bool *bool_)
{
  /* Code For Bool Goes Here */


}

void TypeCheck::visitVoid(Void *void_)
{
  /* Code For Void Goes Here */


}

void TypeCheck::visitArray(Array *array)
{
  /* Code For Array Goes Here */

  array->type_->accept(this);

}

void TypeCheck::visitClassT(ClassT *class_t)
{
  /* Code For ClassT Goes Here */

  class_t->pident_->accept(this);

}

void TypeCheck::visitFun(Fun *fun)
{
  /* Code For Fun Goes Here */

  fun->type_->accept(this);
  fun->listtype_->accept(this);

}

void TypeCheck::visitEVar(EVar *e_var)
{
  /* Code For EVar Goes Here */

  e_var->pident_->accept(this);

}

void TypeCheck::visitELitInt(ELitInt *e_lit_int)
{
  /* Code For ELitInt Goes Here */

  visitInteger(e_lit_int->integer_);

}

void TypeCheck::visitELitTrue(ELitTrue *e_lit_true)
{
  /* Code For ELitTrue Goes Here */


}

void TypeCheck::visitELitFalse(ELitFalse *e_lit_false)
{
  /* Code For ELitFalse Goes Here */


}

void TypeCheck::visitEApp(EApp *e_app)
{
  /* Code For EApp Goes Here */

  e_app->pident_->accept(this);
  e_app->listexpr_->accept(this);

}

void TypeCheck::visitEString(EString *e_string)
{
  /* Code For EString Goes Here */

  visitString(e_string->string_);

}

void TypeCheck::visitENewArray(ENewArray *e_new_array)
{
  /* Code For ENewArray Goes Here */

  e_new_array->type_->accept(this);
  e_new_array->expr_->accept(this);

}

void TypeCheck::visitENewClass(ENewClass *e_new_class)
{
  /* Code For ENewClass Goes Here */

  e_new_class->pident_->accept(this);

}

void TypeCheck::visitEClsMmbr(EClsMmbr *e_cls_mmbr)
{
  /* Code For EClsMmbr Goes Here */

  e_cls_mmbr->expr_->accept(this);
  e_cls_mmbr->pident_->accept(this);

}

void TypeCheck::visitEClsMthd(EClsMthd *e_cls_mthd)
{
  /* Code For EClsMthd Goes Here */

  e_cls_mthd->expr_->accept(this);
  e_cls_mthd->pident_->accept(this);
  e_cls_mthd->listexpr_->accept(this);

}

void TypeCheck::visitNull(Null *null)
{
  /* Code For Null Goes Here */


}

void TypeCheck::visitEIndexAcc(EIndexAcc *e_index_acc)
{
  /* Code For EIndexAcc Goes Here */

  e_index_acc->pident_->accept(this);
  e_index_acc->expr_->accept(this);

}

void TypeCheck::visitECast(ECast *e_cast)
{
  /* Code For ECast Goes Here */

  e_cast->pident_->accept(this);
  e_cast->expr_->accept(this);

}

void TypeCheck::visitNeg(Neg *neg)
{
  /* Code For Neg Goes Here */

  neg->expr_->accept(this);

}

void TypeCheck::visitNot(Not *not_)
{
  /* Code For Not Goes Here */

  not_->expr_->accept(this);

}

void TypeCheck::visitEMul(EMul *e_mul)
{
  /* Code For EMul Goes Here */

  e_mul->expr_1->accept(this);
  e_mul->mulop_->accept(this);
  e_mul->expr_2->accept(this);

}

void TypeCheck::visitEAdd(EAdd *e_add)
{
  /* Code For EAdd Goes Here */

  e_add->expr_1->accept(this);
  e_add->addop_->accept(this);
  e_add->expr_2->accept(this);

}

void TypeCheck::visitERel(ERel *e_rel)
{
  /* Code For ERel Goes Here */

  e_rel->expr_1->accept(this);
  e_rel->relop_->accept(this);
  e_rel->expr_2->accept(this);

}

void TypeCheck::visitEAnd(EAnd *e_and)
{
  /* Code For EAnd Goes Here */

  e_and->expr_1->accept(this);
  e_and->expr_2->accept(this);

}

void TypeCheck::visitEOr(EOr *e_or)
{
  /* Code For EOr Goes Here */

  e_or->expr_1->accept(this);
  e_or->expr_2->accept(this);

}

void TypeCheck::visitPlus(Plus *plus)
{
  /* Code For Plus Goes Here */


}

void TypeCheck::visitMinus(Minus *minus)
{
  /* Code For Minus Goes Here */


}

void TypeCheck::visitTimes(Times *times)
{
  /* Code For Times Goes Here */


}

void TypeCheck::visitDiv(Div *div)
{
  /* Code For Div Goes Here */


}

void TypeCheck::visitMod(Mod *mod)
{
  /* Code For Mod Goes Here */


}

void TypeCheck::visitLTH(LTH *lth)
{
  /* Code For LTH Goes Here */


}

void TypeCheck::visitLE(LE *le)
{
  /* Code For LE Goes Here */


}

void TypeCheck::visitGTH(GTH *gth)
{
  /* Code For GTH Goes Here */


}

void TypeCheck::visitGE(GE *ge)
{
  /* Code For GE Goes Here */


}

void TypeCheck::visitEQU(EQU *equ)
{
  /* Code For EQU Goes Here */


}

void TypeCheck::visitNE(NE *ne)
{
  /* Code For NE Goes Here */


}


void TypeCheck::visitListTopDef(ListTopDef *list_top_def)
{
  for (ListTopDef::iterator i = list_top_def->begin() ; i != list_top_def->end() ; ++i)
  {
    (*i)->accept(this);
  }
}

void TypeCheck::visitListArg(ListArg *list_arg)
{
  for (ListArg::iterator i = list_arg->begin() ; i != list_arg->end() ; ++i)
  {
    (*i)->accept(this);
  }
}

void TypeCheck::visitListClassBlockDef(ListClassBlockDef *list_class_block_def)
{
  for (ListClassBlockDef::iterator i = list_class_block_def->begin() ; i != list_class_block_def->end() ; ++i)
  {
    (*i)->accept(this);
  }
}

void TypeCheck::visitListStmt(ListStmt *list_stmt)
{
  for (ListStmt::iterator i = list_stmt->begin() ; i != list_stmt->end() ; ++i)
  {
    (*i)->accept(this);
  }
}

void TypeCheck::visitListItem(ListItem *list_item)
{
  for (ListItem::iterator i = list_item->begin() ; i != list_item->end() ; ++i)
  {
    (*i)->accept(this);
  }
}

void TypeCheck::visitListType(ListType *list_type)
{
  for (ListType::iterator i = list_type->begin() ; i != list_type->end() ; ++i)
  {
    (*i)->accept(this);
  }
}

void TypeCheck::visitListExpr(ListExpr *list_expr)
{
  for (ListExpr::iterator i = list_expr->begin() ; i != list_expr->end() ; ++i)
  {
    (*i)->accept(this);
  }
}


void TypeCheck::visitInteger(Integer x)
{
  /* Code for Integer Goes Here */
}

void TypeCheck::visitChar(Char x)
{
  /* Code for Char Goes Here */
}

void TypeCheck::visitDouble(Double x)
{
  /* Code for Double Goes Here */
}

void TypeCheck::visitString(String x)
{
  /* Code for String Goes Here */
}

void TypeCheck::visitIdent(Ident x)
{
  /* Code for Ident Goes Here */
}






void TypeCheck::check(Visitable *v)
{
	if (state != State::initialized) {
		throw std::runtime_error("already initialized");
	}
	state = State::buildInfo;
	v->accept(this);
	state = State::checkType;
	v->accept(this);
}